;+
; NAME:
;
;   produce_field_line
;
; PURPOSE:
;
;   Creates the field lines corresponding to the VIP + CANSHEET
;   model of the jovian magnetic field structure.
;
; CALLING SEQUENCE:
;
;   produce_field_line, sc_samples
;
; INPUTS:
;
;   sc_samples: Matrix with three column vectors corresponding to r, theta and phi location of the
;   spacecraft in Jovigraphic SIII Right handed coordinates where theta is given in co-latitude.
;   
; KEYWORD ARGUMENTS:
;   create_maps: this keyword will create csv files that contain the coordinate information for different field lines
;   trajectory_csv: this keyword saves a csv file that contains the spacecraft location, magnetic equator location and centrifugal equator location
;   
;   
; OUTPUTS:
;
;   The output of this code is two text files.
;   One containing the spacecraft location as well as the location of the Centrifugal Equator in six column vectors
;   The second containing the coordinates of the field line
;
; PROCEDURE:
;
;   Calls calculate_footpoint which follows a set of Legendre polynomials to trace the magnetic field of the planet.
;   The code will "walk" along a field line untill it hits the planets surface at which point it will start back at the
;   spacecraft location and walk the opposite direction till once again hitting the surface. At this points, it combines the
;   two coordinate matrices and finds the location of the centrifugal equator
;
; EXAMPLE:
;   produce_field_line, csv_values, /create_maps, /trajectory_csv
;   
; NOTES:
;
;
; Created by Kaleb Bodisch`

forward_function calculate_footpoint

pro produce_field_line, sc_samples, create_maps = create_maps, trajectory_csv = trajectory_csv

Field_line_maps = make_array(4, 1)
trajectories = make_array(10, 1)
start = 0

a=0
for i = start , n_elements(sc_samples[0,*])-1 do begin
     r_sc=(sc_samples[1,*])[i]
     theta_sc=(sc_samples[2,*])[i]
     phi_sc=(sc_samples[3,*])[i]
     decimal_date = (sc_samples[0,*])[i]
     
     print, i ;just keeps track of how far through field line model.

    ;uncomment below if you want the field map to correspond to fields that have spin and centrifugal axis alligned.
    ;phi_sc =-292


    model    = 'CANSheet'
    lower    = [0.001,0.001,0.003]
    t_J2000  = 1

    rho_sc = r_sc *SIN(theta_sc*!PI/180d)     ;calculate rho_sc
    z_sc =r_sc * COS(theta_sc*!PI/180d)       ;calculate z_sc

    ;two calls to footpoint, first call walks in one direction from spacecraft to planet, the second walks the opposite direction to planet
    trc = calculate_footpoint(r_sc, theta_sc, phi_sc, model, [lower,0.1], /RVARY, /LOG, /FULL)
    trc_rev = calculate_footpoint(r_sc, theta_sc, phi_sc, model, [lower,0.1], /RVARY, /LOG, /FULL, /REVDIR)
    full_trc = [trc_rev,trc] ; The matrices don't line up perfectly - allows for line to be drawn to s/c location
  
    rs = full_trc(*,0)
    thetas = full_trc(*,1)
    phis = full_trc(*,2)
    bmin = full_trc(*,3)
  

    ;create field line file, with a 1 bit designating where the field line trace originated
    flag_array = make_array(n_elements(rs))
    flag_array[0] = 1
    if keyword_set(create_maps) then begin
      append= transpose([[flag_array],[rs],[thetas],[phis]])
      Field_line_maps = [[field_line_maps],[append]]
    endif
    
  
    rs = rs[sort(thetas)]           ; sort according to theta (0-180) so that we can plot one continues line instead of having weird cross line
    phis = phis[sort(thetas)]
    bmin = bmin[sort(thetas)]
    thetas = thetas[sort(thetas)]

   ;the commented out code below will cut off the field line at the equator     
   ;ind = WHERE(zzzzzz gt 0)
   ;rrrrrr = rrrrrr(ind)
   ;zzzzzz = zzzzzz(ind)
 
    ;rho and z values of the different field line positions
    rrrrrr = rs * sin(thetas*!PI/180d)
    zzzzzz = rs * COS(thetas*!PI/180d)


    ;set bmin to large number when equal to zero. This is okay since this is only used when field is 0(artifact of starting field line production). Values only used to find mag equator.
    ;this could be done in a more succinct way. 
    bmin[where(bmin eq 0)] = 50000000000000000 
    bmin_loc = where(bmin eq min(bmin))
    bmin_loc = bmin_loc[0]

    ;the calcs between these comments calc ceq values
    ceq_loc = where( rrrrrr eq max(rrrrrr))              ;location of the centrifugal equator corresponds to where the rho value of the field line is largest
    theta_rot = asin(zzzzzz[ceq_loc]/rrrrrr[ceq_loc])
 

    if keyword_set(trajectory_csv) then begin
      append_traj= transpose([[decimal_date],[r_sc],[theta_sc],[phi_sc], [rs[bmin_loc]],[[thetas[bmin_loc]]],[[phis[bmin_loc]]], [rs[ceq_loc]],[[thetas[ceq_loc]]],[[phis[ceq_loc]]]])
      trajectories = [[trajectories],[append_traj]]
    endif


    ;calculate the theoretical(dipole) magnetic and centrifugal equator locations.
    alpha = asin(sin(9.6d*!dPI/180d)*sin((phi_sc - (360.-292.))*!PI/180d))*(180d/!PI)
    theta_mag = 90 - alpha
    theta_ceq = theta_mag + alpha/3
    mag_tilt = theta_mag -90
    ceq_tilt = theta_ceq -90

    ;theoretical location of magnetic equator
    r_mag = [0,60*cos(mag_tilt*!PI/180d)]
    z_mag = [0,60*sin(mag_tilt*!PI/180d)]
    ;theoretical location of centrifugal equator
    r_ceq = [0,60*cos(ceq_tilt*!PI/180d)]
    z_ceq = [0,60*sin(ceq_tilt*!PI/180d)]

  endfor
  
  if keyword_set(create_maps) then begin
     field_line_maps = field_line_maps[*, 1:-1]
     write_csv, 'Field_Line_Map_ephemeris.csv', Field_line_maps, header = ["flag","Radius","Theta","Phi"]
  endif
  
  if keyword_set(trajectory_csv) then begin
    trajectories = trajectories[*, 1:-1]
    write_csv, 'Spacecraft_trajectory_vip4_can_V2.csv', trajectories, header = ["Time","Radius_sc","Theta_sc","Phi_sc","Radius_mag","Theta_mag","Phi_mag","Radius_Cent","Theta_cent","Phi_cent"]
  endif
  

end


;+
; NAME:
;
;   calculate_footpoint
;   
; PURPOSE:
;
;   This function calculates the magnetic surface footpoint of a
;   location around Jupiter in System III coordinates.  It can also
;   return the trace of a magnetic field line.
;
; CALLING SEQUENCE:
;
;   fp = FOOTPOINT(RR, Theta, Phi, Model, InStepSize, [Time])
;
; INPUTS:
;
;   RR:         Radial distance from Jupiter's center to location to map.
;                 Must be given in Jovian radii.
;   Theta:      The co-latitude of location to map (0 to 180 degrees).
;   Phi:        The east longitude of location to map (0 to 360 degrees).
;   Model:      The name of the Jovian magnetic field model to use to
;                 trace the magnetic field.  Currently supports 'VIP4',
;                 'VIT4', 'VIPAL', and 'Dipole'.
;   InStepSize: Either a scalar or a two element array.  If the step size
;                 does not vary, provide a scalar.  If it does, provide a
;                 two element array of the form [minStepSize, maxStepSize].
;                 If two element array given, but neither keyword BVARY nor
;                 RVARY are set, then the constant value minStepSize will
;                 be used.  All step sizes must be given in Jovian radii.
;
; OPTIONAL INPUTS:
;
;   Time:  The 'Khurana' model requires a time in J2000 seconds.  Can use
;            the ctimer or doyToJ2000 functions to obtain this time.
;
; KEYWORD PARAMETERS:
;
;   FULL:        If set, this function will return all coordinates of the
;                  traced magnetic field line instead of just the final
;                  surface footpoint.
;   REVDIR:      If set, the direction along the magnetic field line is
;                  reversed.  The default is to initially trace towards
;                  the planet.
;   BVARY:       If set, the step size will vary between minStepSize and
;                  maxStepSize relative to the magnetic field strength.
;   RVARY:       If set, the step size will vary between minStepSize and
;                  maxStepSize relative to the radial distance to Jupiter's
;                  center.
;   RMIN:        If keyword RVARY is set, then this gives the radial distance,
;                  in Jovian radii, where the step size is equal to minStepSize.
;                  If the radial distance goes below RMIN, then the step size can
;                  go below minStepSize.  (Default: 1)
;   RMAX:        If keyword RVARY is set, then this gives the radial distance,
;                  in Jovian radii, where the step size is equal to maxStepSize.
;                  If the radial distance goes above RMAX, then the step size can
;                  go above maxStepSize.  (Default: 40)
;   BMIN:        If keyword BVARY is set, then this gives the magnetic field
;                  strength, in nanoTesla, where the step size is equal to
;                  maxStepSize.  If the magnetic field strength goes below BMIN,
;                  then the step size can go above maxStepSize.
;                  (Default: Magnetic Field Strength at Initial Location)
;   BMAX:        If keyword BVARY is set, then this gives the magnetic field
;                  strength, in nanoTesla, where the step size is equal to
;                  minStepSize.  If the magnetic field strength goes above BMAX,
;                  then the step size can go below minStepSize. (Default: 100000)
;   LINEAR:      If set, then the step size will scale linearly between minStepSize
;                  and maxStepSize. (If BVARY is set, LINEAR is default)
;   LOGARITHMIC: If set, then the step size will scale logarithmically between
;                  minStepSize and maxStepSize.
;                  (If RVARY is set, LOGARITHMIC is default)
;   INPLANET:    If set, then the initial RR is less than the final radius, generally
;                  the planet's surface.
;   FINALR:      The distance, in Jovian radii, to the final footpoint location.
;                  If set to 0, then the oblate spheroid surface is used.
;                  (Default: 0)
;   EQUATOR:     If set, then the field trace will go away from the planet until Br
;                  changes direction indicating the magnetic equator or current sheet.
;
; OUTPUTS:
;
;   A three element array of the form [rr, theta, phi] with the System III
;   coordinate where the magnetic field line that passes through the initial
;   location crosses the Jovian surface (an oblate spheroid).  As with the
;   input coordinates, rr is in Jovian radii, theta in colatitude degrees,
;   and phi in east longitude degrees.
;
; OPTIONAL OUTPUTS:
;
;   If keyword FULL is set, then the output is a 2D array with all the locations
;   of the magnetic field line between the initial location and the final surface
;   footpoint.  The array is Nx3, where N is the number of steps of the form
;   [stepNum, coordinate].  The array [*,0] gives all rr.  The
;   array [*,1] gives all theta.  And the array [*,2] gives all phi.  The array
;   [stepNum, *] gives the three element coordinate array for the given stepNum.
;
; PROCEDURE:
;
;   This function uses Euler's method to trace along vectors of the magnetic field
;   with a variable step size.  The step size varies in one of three ways:
;     Constant - If inStepSize is a single scalar or neither BVARY nor RVARY are set,
;                  then the step size will be constant throughout.
;     Linear   - If linear scaling is chosen, the step size will vary according to:
;                  stepSize = slope*(currentValue-minValue) + minStepSize, where
;                  slope = (maxStepSize-minStepSize)/(maxValue - minValue)
;                  For BVAR, minValue and maxValue are swapped.
;     Logarithmic - If logarithmic scaling is chosen, the step size will vary
;                  according to:
;                  stepSize = minStepSize * 10^(kk*(currentValue-minValue)), where
;                  kk = ALOG10(maxStepSize/minStepSize)/(maxValue - minValue)
;                  For BVAR, minValue and maxValue are swapped.
;
;   The required functions and procedures are:
;     jovMagField
;
;   If keyword FINALR not specified or set to 0, this function requires:
;     jovSurface
;
;   The models 'VIP4', 'VIT4', and 'VIPAL' require:
;     getSchmidtCoefs
;     getLegendre
;     dPdtheta
;
;   The 'Khurana' model requires:
;     kk_2009
;
; EXAMPLE:
;
;   To find the footpoint of the location at 3 Jovian radii, -70 degrees latitude,
;   and 30 degrees longitude using the 'VIPAL' model and a constant step size of 0.001,
;   use:
;
;     fp = footpoint(3, 160, 30, 'VIPAL', 0.001)
;     fpR     = fp[0]
;     fpTheta = fp[1]
;     fpPhi   = fp[2]
;
;
;   To find the footpoint from location [10, 20, 0] using the 'VIT4' model while
;   varying the step size logarithmically according to distance, with maximum step
;   size (0.1) happening at apojove of 38 Jovian radii and minimum (0.001) at
;   1 Jovian radius, use:
;
;     fp = footpoint(10, 20, 0, 'VIT4', [0.001, 0.1], /RVARY, RMAX=38, /LOG)
;
;
;   To find all coordinates of an entire magnetic field line that extends to an
;   equatorial radius of 5 Jovian Radii, while varying the step size linearly
;   according to magnetic field strength, with the model 'VIP4', use:
;
;     fp1 = footpoint(5, 90, 0, 'VIP4', [0.0001, 0.01], /BVARY, /LIN, /FULL)
;     fp2 = footpoint(5, 90, 0, 'VIP4', [0.0001, 0.01], /BVARY, /LIN, /FULL, /REV)
;
;   To extend a point on the jovian surface (say 70 degrees latitude and 180
;   degrees longitude) out onto a spherical surface of one Jovian radius,
;   with the 'Khurana' model at the day of year 2007/103 07:27:30 (perijove 17 of
;   Juno), use:
;
;     surfaceR = jovSurface(20)
;     time = doyToJ2000(2007, 103, 7, 27, 30)
;     fp = footpoint(surfaceR, 20, 180, 'Khurana', 0.0001, time, /INPLANET, FINALR=1)
;
;   To convert to xyz coordinates, assuming full trace in spherical coordinates
;   stored in variable fp, use:
;
;     fpXYZ = [[fp[*,0] * SIN(fp[*,1]/!radeg) * COS(fp[*,2]/!radeg)], $
;              [fp[*,0] * SIN(fp[*,1]/!radeg) * SIN(fp[*,2]/!radeg)], $
;              [fp[*,0] * COS(fp[*,1]/!radeg)]]
;     fpX = fpXYZ[*,0]
;     fpY = fpXYZ[*,1]
;     fpZ = fpXYZ[*,2]
;
;
; MODIFICATION HISTORY:
;   Written by: Drake A. Ranquist, 12 Jun 2013
;     Added EQUATOR keyword.  26 May 2015
;     modified by Kaleb Bodisch 30 May 2017
;-

function calculate_footpoint, rr, theta, phi, model, inStepSize, time, $
  FULL=full, REVDIR=revDir, RVARY=rvary, $
  RMIN=Rmin, RMAX=rmax, BMIN=Bmin, BMAX=Bmax, $
  LINEAR=lin, LOGARITHMIC=log, $
  INPLANET=inPlanet, FINALR=finalR, MAXR=maxR, $
  GNM=Gnm, HNM=Hnm, CENTEQ=CentEq, BeB=BeB

  ;-------------Set Defaults------------------
  IF NOT KEYWORD_SET(full) THEN full=0
  IF NOT KEYWORD_SET(revDir) THEN revDir=0
  IF NOT KEYWORD_SET(rvary) THEN rvary=0
  IF NOT KEYWORD_SET(Rmin) THEN Rmin=1
  IF NOT KEYWORD_SET(Rmax) THEN Rmax=40
  ;Bmin is set to Binit as default later in code
  IF NOT KEYWORD_SET(bmax) THEN Bmax=100000
  IF NOT KEYWORD_SET(lin) THEN lin=0
  IF NOT KEYWORD_SET(log) THEN log=0
  IF NOT KEYWORD_SET(inPlanet) THEN inPlanet=0
  IF NOT KEYWORD_SET(finalR) THEN finalR=0
  IF NOT KEYWORD_SET(maxR) THEN maxR=100




  ; Logan - Check that code is stepping in the right direction to find Centrifugal equator
  DELVAR, WRONG_DIR
  WRONG_DIR_LABEL:
  IF ISA(WRONG_DIR) THEN BEGIN
    IF revDir EQ 0 THEN revDir = 1 ELSE revDir = 0
  ENDIF

  ;---Set Constants---
  DEG2RAD = !Pi/180

  ;-------Prepare Full Trace Array-------
  IF full THEN BEGIN
    ;Array will automatically increase in size if more room is needed
    traceSize = 10000l
    nn = 0
    tracePos = DBLARR(traceSize, 4)
    ;Store initial position
    tracePos[nn,0:2] = [rr, theta, phi]
    nn++
  ENDIF



  ;---------Can't Scale Both Linearly and Logarithmically--------
  IF lin AND log THEN BEGIN
    print, "Can't scale both linearly and logarithmically!"
    print, "If vary by B, will scale linearly."
    print, "If vary by R, will scale logramithically."
    lin = 0
    log = 0
  ENDIF

  ;----Can't do both BeB and CentEq call simultaneously----
  ; Logan 11/19

  ;------Set Minimum and Maximum Step Sizes------
  IF N_ELEMENTS(inStepSize) gt 1 THEN BEGIN
    minStepSize = MIN(inStepSize)
    maxStepSize = MAX(inStepSize)
  ENDIF ELSE BEGIN
    minStepSize = inStepSize
    maxStepSize = inStepSize
  ENDELSE



  ;---Find Direction to Trace Field Toward Planet---

  Binitial = jovMagField(rr, theta, phi, model, time)
  IF Binitial[0] gt 0 THEN dir = -1 ELSE dir = 1
  IF revDir THEN dir *= -1

  ;--------------Find Initial Field Magnitude-----------------
  IF NOT KEYWORD_SET(Bmin) THEN BEGIN
    Bmin = SQRT(Binitial[0]^2 + Binitial[1]^2 + Binitial[2]^2)
  ENDIF

  B_old = Bmin



  ;----Find Slope or Rate for Varying Step Size by Radial Distance----
  IF rvary THEN BEGIN
    ;----Default to Scale Logarithmically----
    IF NOT (lin AND log) THEN log=1
    IF lin THEN BEGIN
      slope = (maxStepSize-minStepSize)/DOUBLE(Rmax - Rmin)
    ENDIF ELSE BEGIN
      kk = ALOG10(maxStepSize/DOUBLE(minStepSize))/(Rmax - Rmin)
    ENDELSE
  ENDIF

  ;---Prepare Temporary Variables of While Loop---
  nextR = rr
  nextTheta = theta
  nextPhi = phi
  cont = 1

  ;---Calculate initial distance from spin pole. Used for the CentEq keyword later. Set up counter (Logan 10/12/16)---
  r_cyl_old = nextR * SIN(NextTheta*DEG2RAD)
  nct = 0 ; nct also used for B field


  ;---------Keep Tracing Field Until Surface Is Hit-----------
  WHILE cont DO BEGIN

    ;------Find Magnetic Field Vector and Strength-------
    Bfield = jovMagField(nextR, nextTheta, nextPhi, model, time)
    Bmag = SQRT(Bfield[0]^2 + Bfield[1]^2 + Bfield[2]^2)

    ;tracepos[nn,3] = Bmag
    tracepos[nn,3] =Bmag
   ; IF Bfield[0]*dir lt 0 then tracepos[nn,3] =1 else tracepos[nn,3] = 0
        
    ;-------Vary the Step Size if Necessary-------
    stepFactor = minStepSize
    IF rvary THEN BEGIN
      IF lin THEN BEGIN
        stepFactor = slope*(nextR-Rmin) + minStepSize
      ENDIF ELSE IF log THEN BEGIN
        IF nextR GT Rmax THEN BEGIN
          Rmax += 10
          kk = ALOG10(maxStepSize/DOUBLE(minStepSize))/(Rmax - Rmin)
        ENDIF
        stepFactor = minStepSize * 10^(kk*(nextR-Rmin))
      ENDIF
    ENDIF

    ;--Save old values before stepping for CentEq calculation (Logan 10/12/16)--
    phi_old   = nextPhi
    z_Old     = nextR * COS(nextTheta*DEG2RAD)
    
    ;hardcoded larger step size (low res field
    ;stepFactor = .06
    ;--Vector of Correct Direction and Step Size for Next Step--
    stepVector = dir * Bfield/Bmag * ABS(stepFactor)

    ;---------Calculate the Next Position in Spherical Coordinates-----------
    ;nextR must be last because used for calculations of nextPhi and nextTheta
    nextPhi += stepVector[2]/(DEG2RAD * nextR * SIN(nextTheta*DEG2RAD))
    nextTheta += stepVector[1]/(DEG2RAD * nextR)
    nextR += stepVector[0]


    ;-------Add Next Position to the Full Trace Array----------
    IF full THEN BEGIN

      tracePos[nn,0:2] = [nextR, nextTheta, nextPhi]
      nn++
      ;------Increase Array Size if Full-------
      IF nn ge traceSize THEN BEGIN
        newArr = DBLARR(9*traceSize, 4)
        tracePos = [tracePos, newArr]
        traceSize *= 10
      ENDIF
    ENDIF

    cont = nextR gt jovSurface(nextTheta)  ;if trace location smaller than than location of surface, exit loop

  ENDWHILE



  ;----Return the Full Magnetic Field Line Trace----
  IF full THEN BEGIN
    return, tracePos[0:nn-1, *]
  ENDIF


  ;----Return the Footpoint on the Surface----
  return, [nextR, nextTheta, nextPhi]



END

FUNCTION CAN_SHEET, Pos
  ;Magentic field from a current sheet based on
  ;the Connerney, Acuna and Ness model
  ;Pos(0) is radius, in the same units as the parameter, Rj.
  ;Pos(1) is latitude, _not_ co-latitude in degrees.
  ;Pos(2) is West longitude (i.e. System III longitude)
  ;This coordinate system is right handed, but the
  ;       unit vectors in the latitude and longitude
  ;       directions both point anti-parallel to  the
  ;       traditional theta and phi of spherical coordinates

  r0 = 5.0
  r1 = 50.0
  d = 2.5
  c = 2.25E-3
  xt = 9.6*!PI/180.
  xp = 202.*!PI/180.

  B = DBLARR(7)
  r = Pos(0)
  theta = !PI*(90-Pos(1))/180.
  phi = !PI*(360-Pos(2))/180.
 ;print, pos

  
  xp = 2*!PI-xp
  ct=COS(xt)
  st=SIN(xt)
  cp=COS(xp)
  sp=SIN(xp)

  x1 = r*SIN(theta)*cos(phi)
  y1 = r*SIN(theta)*sin(phi)
  z1 = r*COS(theta)

  x2 = cp*ct*x1 + sp*ct*y1 - st*z1
  y2 = -sp*x1 + cp*y1
  z2 = st*cp*x1 + st*sp*y1 + ct*z1

  rr = SQRT(x2^2+y2^2)
  pp = ATAN(y2,x2)
  IF pp LT 0 THEN phi = phi + 2*!PI
  zz = z2
  IF rr LT r0 THEN BEGIN
    F1 = SQRT( (zz-d)^2 + r0^2 )
    F2 = SQRT( (zz+d)^2 + r0^2 )
    F3 = SQRT( zz^2 + r0^2 )
    Br = 0.5*rr*(1/F1 - 1/F2)
    Bz = 2*d/F3 - 0.25*rr^2*( (zz-d)/F1^3 - (zz+d)/F2^3 )
  END ELSE BEGIN
    F1 = SQRT( (zz-d)^2 + rr^2 )
    F2 = SQRT( (zz+d)^2 + rr^2 )
    Br = (F1 - F2 + 2*D)/rr
    IF (ABS(zz) GE D) AND (zz LT 0) THEN Br = (F1 - F2 - 2*D)/rr
    IF (ABS(zz) LT D) THEN Br = (F1 - F2 + 2*zz)/rr
    Br = Br - 0.25*r0^2*rr*(1/F1^3 - 1/F2^3)
    Bz = 2*d/SQRT(zz^2+rr^2) - 0.25*r0^2*( (zz-d)/F1^3 - $
      (zz+d)/F2^3 )
  END

  F1 = SQRT( (zz-d)^2 + r1^2 )
  F2 = SQRT( (zz+d)^2 + r1^2 )
  F3 = SQRT( zz^2 + r1^2 )
  Br2 = 0.5*rr*(1/F1 - 1/F2)
  Bz2 = 2*d/F3 - 0.25*rr^2*( (zz-d)/F1^3 - (zz+d)/F2^3 )

  Bcy = [Br-Br2,0,Bz-Bz2]
  Bcy(1) = SIN(pp)*Bcy(0)
  Bcy(0) = COS(pp)*Bcy(0)

  B1 = cp*ct*Bcy(0) - sp*Bcy(1) + st*cp*Bcy(2)
  B2 = sp*ct*Bcy(0) + cp*Bcy(1) + st*sp*Bcy(2)
  B3 =   -st*Bcy(0)                + ct*Bcy(2)

  cth = COS(theta)
  sth = SIN(theta)
  cph = COS(phi)
  sph = SIN(phi)

  B(0) = sth*cph*B1 + sth*sph*B2 + cth*B3
  B(1) = cth*cph*B1 + cth*sph*B2 - sth*B3
  B(2) = -sph*B1 + cph*b2

  B(1) = -1.*B(1)
  B(2) = -1.*B(2)

  B(3)=B(0)*sth*cph + B(1)*cth*cph - B(2)*sph
  B(4)=B(0)*sth*sph + B(1)*cth*sph + B(2)*cph
  B(5)=B(0)*cth - B(1)*sth
  B(6)=sqrt(B(0)*B(0)+B(1)*B(1)+B(2)*B(2) )
  B = B*c
  bb=sqrt(B(3)*B(3)+B(4)*B(4)+B(5)*B(5) )
  db=abs((bb-B(6))/B(6))
  if db gt 1e-2 then print,'can',pos(0),bb,B(6),db
  RETURN,B
END


;+
; NAME:
;
;   JOVMAGFIELD
;
; PURPOSE:
;
;   This function returns the radial, theta, and phi components of the magnetic
;   field at a specific point given a Jovian magnetic field model.
;
; CALLING SEQUENCE:
;
;   Bfield = JOVMAGFIELD(R, InTheta, InPhi, Model)
;   Br = Bfield[0]
;   Btheta = Bfield[1]
;   Bphi = Bfield[2]
;
; INPUTS:
;
;   R:       Radial coordinate of desired position (in Jupiter Radii)
;   INTHETA: Co-latitude coordinate of desired position (in degrees unless
;             keyword INRAD set)
;   INPHI:   East longitude coordinate of desired position (in degrees unless
;             keyword INRAD set)
;   INMODEL: Name of the internal field model to use.  Only models supported by
;             getSchmidtCoefs.pro allowed.  At first writing, only 'VIP4', 'VIT4',
;             'VIPAL', and 'Dipole'. Now supports the 'Khurana' model.  (Default: 'VIPAL')
;
; OPTIONAL INPUTS:
;
;   TIME:    The J2000 time in seconds.  Necessary with the 'Khurana' model.
;              (Default: Use ctimeOfDPLT to make dipole point towards noon.)
;
; KEYWORD PARAMETERS:
;
;   INRAD:         If set, InTheta and InPhi are given in radians rather than degrees.
;   CURRENT_SHEET: Apply a current sheet to the selected model.  Currently only supports
;                    'CANSheet', which is the Connerney, Acuna, and Ness Current Sheet model.
;                    Do not apply a current sheet to 'Khurana' model because it already
;                    applies a current sheet.
;
; OUTPUTS:
;
;   A three index array with the radial, theta, and phi components of the magnetic
;   field in nanotesla at the requested position and using the given model.  If above
;   one of the poles, the phi component of the magnetic field will be 0.
;
; PROCEDURE:
;
;   Solves for the magnetic field using spherical harmonics with Schmidt quasi-
;   normalized Legendre functions.
;
;   The following functions are required to run:
;     getSchmidtCoefs
;     getLegendre
;     dPdtheta
;     kk_2009
;     FBfield -> CAN_SHEET
;     ctimeOfDPLT
;
; EXAMPLE:
;
;   To get the magnetic field at 2 Jupiter Radii, at the equator and 0 degrees
;   longitude, according to the 'VIP4' model, use:
;     Bfield = jovMagField(2, 90, 0, 'VIP4')
;     Br = Bfield[0]
;     Btheta = Bfield[1]
;     Bphi = Bfield[2]
;
;   To add the Connerny current sheet to the 'VIP4' model use:
;     Bfield = jovMagField(2, 90, 0, 'VIP4', CURRENT_SHEET='CANSheet')
;
;   To get the magnitude of the magnetic field at 5 Jupiter radii, -30 degrees latittude,
;   and 45 degrees west longitude, according to the 'VIPAL' model, use:
;     Bfield = jovMagField(5, 120, 315, 'VIPAL')
;     Bmag = SQRT(Bfield[0]^2 + Bfield[1]^2 + Bfield[2]^2)
;
;
; MODIFICATION HISTORY:
;   Written by: Drake A. Ranquist, 7 Jun 2013
;     8 Jul 2013, Added compatibility with the Khurana Field Model
;     29 Sep 2016, Added Connerney, Acuna, and Ness Current Sheet Model
;-
function jovMagField, r, inTheta, inPhi, inModel, time, INRAD=inRad, CURRENT_SHEET=currentSheet

  ;----------------Set Defaults-------------------

  IF NOT KEYWORD_SET(inRad) THEN inRad=0
  IF NOT KEYWORD_SET(currentSheet) THEN currentSheet=''

  ;Make it so that 'CANSheet' can be given as the model
  IF inModel EQ 'CANSheet' AND currentSheet EQ '' THEN BEGIN
    model = 'VIP4'
    currentSheet = 'CANSheet'
  ENDIF ELSE BEGIN
    model = inModel
  ENDELSE

  ;-------Change inTheta and inPhi to Radians------
  theta=inTheta
  phi=inPhi
  IF NOT inRad THEN BEGIN
    theta *= !Pi/180
    phi *= !Pi/180
  ENDIF


  maxN = 4


  ;--Initialize Three B Field Components--
  Br = 0
  Btheta = 0
  Bphi = 0

  ;Perform Summations for All Components of B
  FOR nn=1, maxN DO BEGIN
    radialFactor = r^(-(nn+2.))
    FOR mm=0, nn DO BEGIN

      ;-------------Find Radial B Field Component---------------
      gh = getSchmidtCoefs(model, nn, mm)
      phiFactor = gh[0]*COS(mm*phi) + gh[1]*SIN(mm*phi)
      Pnm = getLegendre(nn, mm, theta, /INRAD)
      Br += (nn+1) * radialFactor * phiFactor * Pnm

      ;--------------Find Theta B Field Component---------------
      dPnmdtheta = dPdtheta(nn, mm, theta, /INRAD)
      Btheta -= radialFactor * phiFactor * dPnmdtheta

      ;----------------Find Phi B Field Component-------------------
      Bphi += mm * radialFactor * (gh[0]*SIN(mm*phi) - gh[1]*COS(mm*phi)) * Pnm
    ENDFOR
  ENDFOR


  ;--------Correct Bphi and Don't Divide by 0!------------
  sinTheta = SIN(theta)
  IF sinTheta ne 0 THEN Bphi /= sinTheta ELSE Bphi=0

  ;--------Apply Curent Sheet Model-------------
  IF currentSheet EQ 'CANSheet' THEN BEGIN
    ;---------Put position in r, latitude, west longitude form
    IF inRad THEN BEGIN
      pos = [r, 90-inTheta*!radeg, 360-inPhi*!radeg]
    ENDIF ELSE BEGIN
      pos = [r, 90-inTheta, 360-inPhi]
    ENDELSE

    ;Get Current Sheet perturbations and convert to nT
    CANB = CAN_SHEET(pos)*1.e5

    ;CAN_SHEET puts theta and phi vectors in opposite direction
    Br += CANB[0]
    Btheta -= CANB[1]
    Bphi -= CANB[2]

  ENDIF

  return, [Br, Btheta, Bphi]

END
;+
; NAME:
;
;   JOVSURFACE
;
; PURPOSE:
;
;   Since Jupiter is an oblate spheroid, this function returns the radial
;   distance from the center of Jupiter to the surface at a given
;   co-latitude.  Returned as a fraction of the equatorial radius (Rj).
;
; CALLING SEQUENCE:
;
;   R_Surface = JOVSURFACE(Theta)
;
; INPUTS:
;
;   Theta:  Co-latitude (0 to 180) in degrees (unless INRAD keyword set)
;
; KEYWORD PARAMETERS:
;
;   INRAD:  If set, Theta is assumed to be in radians.
;   OBLATE: Inverse of the oblateness of Jupiter.  (Default: 15.41)
;   RADIUS: Equatorial radius of Jupiter.  (Default: 1 Jovian Radius)
;
; OUTPUTS:
;
;   Distance from the center of Jupiter to the surface at the given co-latitude.
;
; EXAMPLE:
;
;   To get the ratio of the polar radius to the equatorial radius, use:
;     eqRadius = jovSurface(0)
;
;   To get the distance to surface in km at -30 degrees latitude, use:
;     distance = jovSurface(120, RADIUS=71492)
;
; MODIFICATION HISTORY:
;   Written by: Drake A. Ranquist, 10 Jun 2013
;-

function jovSurface, theta, INRAD=inRad, OBLATE=oblate, RADIUS=radius

  ;-------------Set Defaults------------------
  IF NOT KEYWORD_SET(inRad) THEN inRad=0
  IF NOT KEYWORD_SET(oblate) THEN oblate=15.41
  IF NOT KEYWORD_SET(radius) THEN radius=1

  ;----------Convert theta to Radians-----------
  thisTheta = theta
  IF NOT inRad THEN thisTheta = theta * !Pi/180

  ;-------Solve for the Polar Radius-----------
  polRadius = radius - radius/DOUBLE(oblate)

  ;---Solve for X and Z Coordinates of Surface---
  xx = radius*SIN(thisTheta)
  zz = polRadius*COS(thisTheta)

  ;----Return Distance to Surface----
  return, SQRT(xx^2 + zz^2)

END