;+
; NAME:
; 
;   GETSCHMIDTCOEFS
;
; PURPOSE:
; 
;   This function retrieves the Schmidt quasi-normalized coefficients for
;   three different spherical harmonics models of the Jovian magnetosphere.
;   The available models are 'VIT4', 'VIP4', and 'VIPAL'.  
;
; CALLING SEQUENCE:
; 
;   gh = GETSCHMIDTCOEFS(MODEL, N, M)
;   g = gh[0]
;   h = gh[1]
;
; INPUTS:
;   MODEL: Name of the model
;   N:     Degree of the Schmidt quasi-normalized Legendre function.  N can take
;           on the values between 1 and 4 for the 'VIT4' and 'VIP4' models.  
;           For 'VIPAL', N can be between 1 and 5.
;   M:     Order of the Schmidt quasi-normalized Legendre function (0 to n)
;
; KEYWORD PARAMETERS:
;
;   INGAUSS: If set, the return value will be in gauss rather than nanotesla.
;             10^5 nanotesla = 1 Gauss
;
; OUTPUTS:
;
;   A two element array with the g and h coefficients for the Schmidt
;   quasi-normalized Legendre functions of degree n and order m.  The array is
;   arranged like so: [g, h].  The coefficients are given in nanotesla unless
;   the INGAUSS keyword is set.
;   
; PROCEDURE:
; 
;   Uses a series of case lookup.
;
; EXAMPLE:
;
;   To retrieve the coefficients of degree 3 and order 2 of the 'VIP4' model, use:
;     gh = getSchmidtCoefs('VIP4', 3, 2)
;     g = gh[0]
;     h = gh[1]
;     
;   To retrieve, in Gauss, the coefficients of degree 5 and order 0 
;   of the 'VIPAL' model, use:
;     gh = getSchmidtCoefs('VIPAL', 5, 0, /INGAUSS)
;     g = gh[0]
;     h = gh[1]
;
; MODIFICATION HISTORY:
;   Written by: Drake A. Ranquist, 7 Jun 2013
;-

function getSchmidtCoefs, model, n, m, INGAUSS=inGauss

IF NOT KEYWORD_SET(inGauss) THEN inGauss=0

gh = [0, 0]

CASE model OF
  'VIP4': CASE n OF
    1: CASE m OF
      0: gh = [420543, 0]
      1: gh = [-65920, 24992]
    ENDCASE
    2: CASE m OF
      0: gh = [-5118, 0]
      1: gh = [-61904, -36052]
      2: gh = [49690, 5250]
    ENDCASE 
    3: CASE m OF
      0: gh = [-1576, 0]
      1: gh = [-52036, -8804]
      2: gh = [24386, 40829]
      3: gh = [-17597, -31586]
    ENDCASE   
    4: CASE m OF
      0: gh = [-16758, 0]
      1: gh = [22210, 7557]
      2: gh = [-6074, 40411]
      3: gh = [-20243, -16597]
      4: gh = [6643, 3866]
    ENDCASE
  ENDCASE
ENDCASE

IF inGauss THEN gh /= 100000.

return, gh

END